#ifndef CURSES_H
#define CURSES_H

#ifndef LIBBB_H
typedef unsigned char bool;
#endif

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

typedef unsigned int chtype;
typedef struct WINDOW WINDOW;

#define COLS 80
#define LINES 25

/* Refresh functions */
int wrefresh(WINDOW *win); /* Copy window to virtual screen, and then to physical */
#define refresh() wrefresh(stdscr)
int wnoutrefresh(WINDOW *win); /* Copy window to virtual screen only */

/* Sets attribute of next charactes to be printed */
int wbkgdset(WINDOW *win, int attrs);

/* Creates a new Window */
WINDOW *newwin(int nlines, int ncols, int begin_y, int begin_x);
WINDOW *subwin(WINDOW *win, int nlines, int ncols, int begin_y, int begin_x);
int delwin(WINDOW *win);

/* Moves cursor in window to given position */
int wmove(WINDOW *win, int y, int x);
#define move(y,x) wmove((stdscr),(y),(x))

/* Adds a character or string to the window, and move cursor */
int waddch(WINDOW *win, const chtype ch);
int waddstr(WINDOW *win, const char *str);

/* Reads one character form keyboard */
int wgetch(WINDOW *win);

int werase(WINDOW *win);

/* Standard screen (Window representing entire screen */
extern WINDOW *stdscr;

/* Keypad */
#define keypad(win,bf) do{}while(0);

/* Get current cursor position */
void _getyx(WINDOW *win, int *x, int *y);
#define getyx(win,y,x) _getyx(win, &(y), &(x))

/* Scrolls window */
int wscrl(WINDOW *win, int n);
#define scrollok(win,bf) do{}while(0)
#define scroll(win) wscrl((win),1)

chtype winch(WINDOW *win);
#define A_CHARTEXT 0xff

/* Colors */
int init_pair(int pair, int fg, int bg);
int COLOR_PAIR(int pair);

/* Flash */
int flash(void);

/* No ops */
#define touchwin(x) do{}while(0)
#define use_legacy_coding(x) do{}while(0)

#define has_colors() 1

#define start_color() do{}while(0)

int endwin(void);

WINDOW *initscr(void);
int cbreak(void);
int noecho(void);

/* Line drawing characters */
#define ACS_ULCORNER	((unsigned char) 0xda)
#define ACS_HLINE	((unsigned char) 0xc4)
#define ACS_URCORNER	((unsigned char) 0xbf)

#define ACS_VLINE	((unsigned char) 0xb3)

#define ACS_LTEE	((unsigned char) 0xc3)
#define ACS_RTEE	((unsigned char) 0xb4)

#define ACS_LLCORNER	((unsigned char) 0xc0)
#define ACS_LRCORNER	((unsigned char) 0xd9)

#define ACS_UARROW	((unsigned char) 0x18)
#define ACS_DARROW	((unsigned char) 0x19)

/* Keyboard */
#define KEY_DOWN        0402            /* down-arrow key */
#define KEY_UP          0403            /* up-arrow key */
#define KEY_LEFT        0404            /* left-arrow key */
#define KEY_RIGHT       0405            /* right-arrow key */

#define KEY_NPAGE       0522            /* PgDown */
#define KEY_PPAGE       0523            /* PgUp */

#define KEY_HOME	0406	
#define KEY_END		0550
#define KEY_EOL		0517
#define KEY_DC		0512		/* Delete */
#define KEY_BACKSPACE	0407

#define ERR     (-1)

/* Colors */
#define COLOR_BLACK	0x0
#define COLOR_RED	0x1
#define COLOR_GREEN	0x2
#define COLOR_YELLOW	0x3
#define COLOR_BLUE	0x4
#define COLOR_CYAN	0x6
#define COLOR_WHITE	0x7

#define NCURSES_ATTR_SHIFT       8
#define NCURSES_BITS(mask,shift) ((mask) << ((shift) + NCURSES_ATTR_SHIFT))


#define A_NORMAL        (1UL - 1UL)
#define A_UNDERLINE     NCURSES_BITS(1UL,9)
#define A_REVERSE       NCURSES_BITS(1UL,10)
#define A_DIM           NCURSES_BITS(1UL,12)
#define A_BOLD          NCURSES_BITS(1UL,13)

#define A_COLOR_VALID     NCURSES_BITS(1UL,3)

#endif
