/*
 *  yesno.c -- implements the yes/no box
 *
 *  AUTHOR: Savio Lam (lam836@cs.cuhk.hk)
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif
#include "dialog.h"
#include "udpc_lang.h"
#include "udpc_dialog.h"

/* Careful here: The Yes/No box uses an inverted boolean logic: 1 for No,
 * 0 for yes! */


static void printBoolButton(int button, WINDOW *dialog, int y, int x)
{
    /* Different order of button drawing is needed in order to make sure
       that cursor shows up in correct button... */
    if(button) {
	print_button (dialog, yesBuffer, y, x, FALSE);
	print_button (dialog, noBuffer, y, x + 13, TRUE);
    } else {
	print_button (dialog, noBuffer, y, x + 13, FALSE);
	print_button (dialog, yesBuffer, y, x, TRUE);
    }
    wrefresh (dialog);
}

/*
 * Display a dialog box with two buttons - Yes and No
 */
int
dialog_yesno (const char *title, const char *prompt, int height, int width,
	      int defaultValue)
{
    int i, x, y, key = 0, button = defaultValue;
    WINDOW *dialog;

    /* center dialog box on screen */
    x = (COLS - width) / 2;
    y = (LINES - height) / 2;

#ifdef HAVE_NCURSES
    if (use_shadow)
	draw_shadow (stdscr, y, x, height, width);
#endif
    dialog = newwin (height, width, y, x);
    keypad (dialog, TRUE);

    draw_box (dialog, 0, 0, height, width, dialog_attr, border_attr);
    wbkgdset (dialog, border_attr);
    wmove (dialog, height - 3, 0);
    waddch (dialog, ACS_LTEE);
    for (i = 0; i < width - 2; i++)
	waddch (dialog, ACS_HLINE);
    wbkgdset (dialog, dialog_attr);
    waddch (dialog, ACS_RTEE);
    wmove (dialog, height - 2, 1);
    for (i = 0; i < width - 2; i++)
	waddch (dialog, ' ');

    if (title != NULL) {
	wbkgdset (dialog, title_attr);
	wmove (dialog, 0, (width - strlen (title)) / 2 - 1);
	waddch (dialog, ' ');
	waddstr (dialog, title);
	waddch (dialog, ' ');
    }
    wbkgdset (dialog, dialog_attr);
    print_autowrap (dialog, prompt, width - 2, 1, 3);

    x = width / 2 - 10;
    y = height - 2;
    printBoolButton(button, dialog, y, x);

    while (key != ESC) {
	key = wgetch (dialog);
	switch (key) {
	case 'O':
	case 'o':
	case 'Y':
	case 'y':
	case 'J':
	case 'j':
	    delwin (dialog);
	    return 0;
	case 'N':
	case 'n':
	    delwin (dialog);
	    return 1;

	case TAB:
	case KEY_UP:
	case KEY_DOWN:
	case KEY_LEFT:
	case KEY_RIGHT:
	    button = !button;
	    printBoolButton(button, dialog, y, x);
	    break;
	case ' ':
	case '\n':
	    delwin (dialog);
	    return button ? 1 : 0;
	case ESC:
	    break;
	}
    }

    delwin (dialog);
    return -1;			/* ESC pressed */
}
