/*
 *  udev.c -- creates device nodes on udpcast boot filesystem
 *
 *  AUTHOR: Alain Knaff (alain@knaff.lu)
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include "udpc_dialog.h"
#include "libbb.h"

static int try_create_device(int dirFd, const char *devlink, 
			     int devFd, int isBlock)
{
  char linkTarget[MAXPATHLEN+7];
  const char *devName;
  const char *devNum;
  int ret;
  dev_t dev;
  mode_t mode;
  char *eptr;
  int myMajor;
  int myMinor;

  ret = readlinkat(dirFd, devlink, linkTarget, sizeof(linkTarget));
  if(ret < 0 || ret >= (int) sizeof(linkTarget)) {
    PMSG("Could not readlink %s => %s\n", devlink, strerror(errno));
    return -1;
  }
  linkTarget[ret]='\0';
  devName = bb_basename(linkTarget);
  devNum = bb_basename(devlink);

  myMajor = strtoul(devNum, &eptr, 10);
  myMinor = strtoul(eptr+1, &eptr, 10);
  dev = makedev(myMajor, myMinor);
  if(isBlock)
    mode = S_IFBLK | 0600;
  else 
    mode = S_IFCHR | 0600;
  if(mknodat(devFd, devName, mode, dev) < 0) {
    if(errno != EEXIST)
      PMSG("Could not make node %s => %s\n", devName, strerror(errno));
    return -1;
  }
  return 0;
}

static int try_create_devices(int devFd, const char *dirName, int isBlock)
{
  int dirFd;
  DIR *f = opendir(dirName);
  struct dirent *d;
  int found = 0;

  if(f == NULL) {
    perror("open");
    return -1;
  }

  dirFd = open(dirName, O_RDONLY);

  while( (d=readdir(f)) != NULL) {
    const char *name = d->d_name;
    if(!strcmp(name, ".") || !strcmp(name, ".."))
      /* Skip and "reserved" names in directory */
      continue;

    if(try_create_device(dirFd, name, devFd, isBlock) == 0)
      found = 1;
  }
  closedir(f);
  close(dirFd);
  return found;
}

int udpc_makeDevices(void)
{
  int found = 0;
  int devFd = open("/dev/", O_RDONLY);

  found |= try_create_devices(devFd, "/sys/dev/char", 0);
  found |= try_create_devices(devFd, "/sys/dev/block", 1);
  
  close(devFd);
  return found;
}
