/*
 *  udpc_dialog.c -- series of dialogs for configuring udpcast
 *
 *  AUTHOR: Alain Knaff (alain@knaff.lu)
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
//config:config UDPCDIALOG
//config:	bool "UDPCDIALOG"
//config:	default y
//config:	help
//config:	  Boot Menu for udpcast
//config:
//config:config UDPCDIALOG_USB
//config:	bool "UDPCDIALOG_USB"
//config:	default n
//config:	depends on UDPCDIALOG
//config:	help
//config:	  Udpcdialog with support for storing config on Usb
//config:
//config:config UDPCDIALOG_PXESERVER
//config:	bool "UDPCDIALOG_PXESERVER"
//config:	default n
//config:	depends on UDPCDIALOG_USB
//config:	help
//config:	  Udpcdialog with support for starting a PXE server
//config:
//config:config UDPCDIALOG_VERBOSE
//config:	bool "UDPCDIALOG_VERBOSE"
//config:	default n
//config:	depends on UDPCDIALOG
//config:	help
//config:	  Udpcdialog with more verbose messages

//kbuild:lib-$(CONFIG_UDPCDIALOG)  += dialog_inputbox.o
//kbuild:lib-$(CONFIG_UDPCDIALOG)  += dialog_menubox.o
//kbuild:lib-$(CONFIG_UDPCDIALOG)  += dialog_msgbox.o
//kbuild:lib-$(CONFIG_UDPCDIALOG)  += dialog_util.o
//kbuild:lib-$(CONFIG_UDPCDIALOG)  += dialog_yesno.o
//kbuild:lib-$(CONFIG_UDPCDIALOG)  += udev.o
//kbuild:lib-$(CONFIG_UDPCDIALOG)  += udpc_conffile.o
//kbuild:lib-$(CONFIG_UDPCDIALOG)  += udpc_dialog.o
//kbuild:lib-$(CONFIG_UDPCDIALOG)  += udpc_diskdriver.o
//kbuild:lib-$(CONFIG_UDPCDIALOG)  += udpc_lang.o
//kbuild:lib-$(CONFIG_UDPCDIALOG)  += udpc_netconfig.o
//kbuild:lib-$(CONFIG_UDPCDIALOG)  += udpc_netdriver.o
//kbuild:lib-$(CONFIG_UDPCDIALOG)  += udpc_dev.o
//kbuild:lib-$(CONFIG_UDPCDIALOG)  += udpc_udpc_cfg.o
//kbuild:lib-$(CONFIG_UDPCDIALOG_PXESERVER)  += udpc_pxeServer.o
//kbuild:lib-$(CONFIG_UDPCDIALOG)  += udpc_ui.o
//kbuild:lib-$(CONFIG_UDPCDIALOG)  += udpc_util.o
//kbuild:lib-$(CONFIG_UDPCDIALOG)  += curses.o

//applet:IF_UDPCDIALOG(APPLET(udpc_dialog, BB_DIR_USR_SBIN, BB_SUID_DROP))

//usage:#define udpc_dialog_trivial_usage
//usage:      "[-S stderrfile] [t]"
//usage:#define udpc_dialog_full_usage
//usage:      "Udpcast's interactive dialog system\n\n"
//usage:      "\t-S\tlog stderr messages into this file\n"
//usage:      "\t-t\ttest mode\n"


#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdarg.h>
#include <errno.h>
#include <dirent.h>
#include <sys/klog.h>
#include <ftw.h>
#include <sys/ioctl.h>
/* #include <linux/fs.h>*/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include "dialogExt.h"

#include "udpc_lang.h"
#include "udpc_dialog.h"
#include "udpc_conffile.h"
#include "libbb.h"

struct config udpc_config;

struct configDesc desc[]= {
  { "auto", TYPE_BOOLEAN,	-1, offsetof(struct config, automatic), -1 },
  { "lang", TYPE_NONNULL_STRING,	LANGSIZE, 
    offsetof(struct config, language), -1 },
  { "kbmap", TYPE_NONNULL_STRING,	KMAPSIZE, 
    offsetof(struct config, keymap), -1 },

  { "netauto", TYPE_BOOLEAN,	-1, offsetof(struct config, netAutoconfig), -1 },
  { "netmodule", TYPE_NONNULL_STRING,	MODSIZE,
    offsetof(struct config,netModule), -1},
  { "netmodparm", TYPE_STRING,	MODPARAMSIZE,
    offsetof(struct config, netParam),offsetof(struct config, netParamIsSet)},
  { "defNic", TYPE_STRING,	IFNAMESIZE,
    offsetof(struct config, defNic), -1},

  { "enableDiskmodule", TYPE_BOOLEAN,	-1, offsetof(struct config, enableDiskmodule), -1 },

  { "diskmodule", TYPE_NONNULL_STRING,	MODSIZE,
    offsetof(struct config,diskModule), -1},
  { "diskmodparm", TYPE_STRING,	MODPARAMSIZE,
    offsetof(struct config, diskParam),offsetof(struct config, diskParamIsSet)},

  { "dhcp", TYPE_BOOLEAN,	-1, offsetof(struct config, doDhcp), -1 },

  { "ip", TYPE_IP,	-1, 
    offsetof(struct config, ip), offsetof(struct config, ipIsSet)},
  { "netmask", TYPE_IP,	-1, 
    offsetof(struct config, netmask), offsetof(struct config, netmaskIsSet)},

  { "port", TYPE_INTEGER,	-1, 
    offsetof(struct config, port), offsetof(struct config, portIsSet)},

  { "disk", TYPE_NONNULL_STRING,	DISKNAMESIZE, 
    offsetof(struct config, disk), -1},

  { "umode", TYPE_NONNULL_STRING,	MODESIZE, 
    offsetof(struct config, mode), -1},

  { "compr", TYPE_NONNULL_STRING,	COMPRSIZE, 
    offsetof(struct config, compr), -1},

  { "udpcparam", TYPE_STRING,	UDPCPARAMSIZE, 
    offsetof(struct config, udpcParam), offsetof(struct config, udpcParamIsSet)},
  { "udprparam", TYPE_STRING,	UDPCPARAMSIZE, 
    offsetof(struct config, udprParam), offsetof(struct config, udprParamIsSet)},
  { "udpsparam", TYPE_STRING,	UDPCPARAMSIZE, 
    offsetof(struct config, udpsParam), offsetof(struct config, udpsParamIsSet)},
  { "pxe", TYPE_BOOLEAN,	-1, offsetof(struct config, doPxe), -1 },

  { NULL, -1, -1, -1, -1 }
};

int udpc_haveFloppy=0; /* Do we have a floppy drive? */
#ifdef CONFIG_UDPCDIALOG_USB
int udpc_haveUsb=0; /* Do we have a Usb stick? */
#endif



static const char *mount_floppy_cmd[] = { "mount", "/floppy", NULL };

static int floppyChanged=0;

/* Mounts the floppy, asking the user to insert it if necessary 
 * if askMsg is set, asks user to insert floppy if necessary (no floppy
 * present)
 * if alwaysask is set, always ask, even if a floppy is in drive
 * Returns -2 if user cancelled
 *         -1 if floppy could not be mounted
 */
int udpc_mountFloppy(int alwaysAsk, const char *askMsg)
{
  int r;
  if(alwaysAsk)
    floppyChanged=1;
  while(1) {
    if(alwaysAsk) {
      r = udpc_info(askMsg);
      if(r)
	return -2;
    }
    r = udpc_spawn(-1, -1, mount_floppy_cmd);
    if(r == 0) {
      /* success ! */
      return 0;
    }
    if(!askMsg) {
      return -1;
    }
    if(alwaysAsk) {
      /* display the error */
      udpc_display_error();
    }
    alwaysAsk=1;
  }
}

static const char *umount_floppy_cmd[] = { "umount", "/floppy", NULL };

int udpc_umountFloppy(void)
{
  return udpc_spawn(-1, -1, umount_floppy_cmd);
}

#ifdef CONFIG_UDPCDIALOG_USB
int udpc_mountUsbRo(void)
{
  /* we don't prompt for USB: the one that we booted from is sufficiently
   * large to contain everything we might ever need */
  return system("mountUsb");
}

static int udpc_mountUsbRw(void)
{
  return system("mountUsb rw");
}

int udpc_umountUsb(void)
{
  return system("umountUsb");
}
#endif

int udpc_askBoolean(int title, const char *prompt,
		    int height, int width,
		    int *value, int forceManual)
{
  int r;
  if(*value != -1 && !forceManual)
    return DIALOG_FORWARD;
  r = dialog_yesno(MSG(title), prompt, height, width, (*value == 0));
  dialog_clear();
  if(r < 0)
    return DIALOG_BACK;
  *value = 1-r;
  return DIALOG_FORWARD;
}

static int askSaveFile(int forceManual, int dir UNUSED, void *data UNUSED)
{
  int r;
  int saveMode=0; /* should we also save the mode ? */
#ifdef CONFIG_UDPCDIALOG_USB
  struct stat statbuf;
#endif

  /* only ask for saving if we are in manual mode... */
  if(!forceManual)
    return 0;

  /* if we don't have a removable drive, don't attempt to save config: */
  if(!udpc_haveFloppy 
#ifdef CONFIG_UDPCDIALOG_USB
     && (!udpc_haveUsb  || (stat("/readonly", &statbuf) == 0))
#endif
     )
    return 0;

  while(1) {
    r = dialog_yesno(MSG(TXT_SAVEIT), MSG(TXT_SAVEIT), 5, 50, !0);
    dialog_clear();
    if(r < 0)
      return DIALOG_BACK;
    if(r)
      return DIALOG_FORWARD; /* No save requested */
    r = dialog_yesno(MSG(TXT_SAVEIT), MSG(TXT_SAVEMODE), 5, 70, !0);
    dialog_clear();
    if(r < 0)
      continue;
    saveMode = !r;

    udpc_config.automatic=1;
    if(udpc_haveFloppy) {
      r = udpc_mountFloppy(floppyChanged, MSG(TXT_INSERT_CONFIG_FLOPPY));
      if(r)
	continue;
      if(!saveMode) {
	udpc_config.mode[0] = '\0';
      }
      udpc_saveConfig("/floppy/udpcfg.txt", &udpc_config, desc);
      udpc_umountFloppy();
#ifdef CONFIG_UDPCDIALOG_USB
    } else if(udpc_haveUsb) {
      r = udpc_mountUsbRw();
      if(r) {
	udpc_alert("Error while saving");
	continue;
      }

      if(!saveMode) {
	udpc_config.mode[0] = '\0';
      }
      udpc_saveConfig("/removable/udpcfg.txt", &udpc_config, desc);
      udpc_umountUsb();
#endif
    }
    return DIALOG_FORWARD;
  }
}

static int sentinel(int forceManual UNUSED, int dir, void *data UNUSED)
{
  if(dir == DIALOG_BACK)
    udpc_alert(MSG(TXT_INVALID_INPUT));
  return DIALOG_FORWARD;
}

static configFn_t mainMenu[] = {
  sentinel,
  udpc_uiConfig,
  udpc_configureNetDriver,
  udpc_configureNetSettings,
  udpc_configureDiskDriver,
  udpc_configureUdpcast,
#ifdef CONFIG_UDPCDIALOG_PXESERVER
  udpc_configurePxeServer,
#endif
  askSaveFile,
  NULL
};

/* ===============================================================
 * Main function
 * =============================================================== */

static void mainDialog(void)
{
  int forceManual = 0;
  if(udpc_config.automatic != 1)
    forceManual=1;
  dialogSeries(mainMenu, forceManual, DIALOG_FORWARD, NULL);
}

int dialogSeries(configFn_t *fn, int forceManual, int dir, void *data)
{
  unsigned int slot;
  unsigned int size;
  
  for(size=0; fn[size]; size++);

  for(slot= (dir == DIALOG_FORWARD) ? 0 : size-1 ; slot < size; ) {
    switch(fn[slot](forceManual, dir, data)) {

    case DIALOG_EXIT:
      return DIALOG_FORWARD;

    case DIALOG_SKIP:
      if (dir == DIALOG_FORWARD) {
	slot++;
	break;
      }
      /* FALLTHROUGH */

    case DIALOG_BACK:
      if(slot == 0)
	return DIALOG_BACK;
      else {
	forceManual = 1;
	slot--;
	dir=DIALOG_BACK;
      }
      break;

    case DIALOG_REPEAT:
      forceManual=1;
      dir=DIALOG_FORWARD;
      break;

    case DIALOG_STARTOVER:
      forceManual=1;
      slot=0;
      dir=DIALOG_FORWARD;
      break;

    default:
      slot++;
      dir=DIALOG_FORWARD;
    }
  }
  return DIALOG_FORWARD;
}

static void set_parm(const char *file, int value)
{
    int fd;
    fd = open(file, O_WRONLY);
    if(fd < 0) {
	PMSG( "Could not open %s (%s)\n", file, strerror(errno));
	return;
    }
    dprintf(fd, "%d", value);
    close(fd);
}

int udpc_modprobe(const char *module, const char *params)
{
    return udpc_shell_spawn(-1, -1, "modprobe %s %s", module, params);
}

static const char *cmd[] = { "udp-receiver", "udp-sender" };

static const char *compr[] = {
  "",
  " -p gunzip",
  " -p 'lzop -d'",
  "",
  " -p gzip",
  " -p lzop",
};

static const char *floppyModules[] = {
    "floppy", "nls_cp437","nls_iso8859-1", "vfat"
};

static int systemf(int onlyIfExists, const char *fmt, ...)
  __attribute__ ((format (printf, 2, 3)));
static int systemf(int onlyIfExists, const char *fmt, ...) {
  va_list ap;
  char ch;
  int size;
  int ret;

  va_start(ap, fmt);
  size=vsnprintf(&ch, 1, fmt, ap);
  if(size >= 0) {
    char *bigbuf = alloca(size+1);
    struct stat statBuf;
    char *end;
    vsnprintf(bigbuf, size+1, fmt, ap);
    end = strchr(bigbuf, ' ');
    if(end) {
      *end='\0';
    }
    if(onlyIfExists && stat(bigbuf, &statBuf) < 0) {
      ret=0;
    } else {
      if(end)
	*end=' ';
      ret=system(bigbuf);
    }
  } else {
    ret=-1;
  }
  va_end(ap);
  return ret;
}

static int loadStuffFromRemovable(const char *dir)
{
  int fd;

  int dirFd = open(dir, O_RDONLY);
  if(dirFd < 0) {    
    PMSG("Could not open %s (%s)\n", dir, strerror(errno));
    return -1;
  }

  udpc_loadConfig(dirFd, "udpcfg.txt", &udpc_config, desc, 0);

  /* If floppy flag is set, copy any items (*.o, *.o.gz, *.ko, *.ko.gz and *.tar) from host
   * floppy */
  udpc_loadModulesFromFloppy("/floppy/");

  fd = openat(dirFd, "ipmac.txt", O_RDONLY);
  if(fd >= 0) {
    int out = open("/tmp/ipmac.txt", O_WRONLY|O_CREAT, 0644);
    if(out >= 0) {
      bb_copyfd_eof(fd, out);
      close(out);
    } else
      perror("write /tmp/ipmac.txt file");
    close(fd);
  } else {
    perror("open ipmac.txt file");
  }
  close(dirFd);
  return 0;
}

#ifndef NO_BB

int udpc_dialog_main(int argc, char **argv);
int udpc_dialog_main(int argc, char **argv)
#else
const char *bb_applet_name="udpcdialog";
int main(int argc, char **argv)
#endif
{
  unsigned int i;

  if(argc >= 2 && !strcmp(argv[1], "init")) {
    return !udpc_makeDevices();
  }

  systemf(1, "/predialog");
  systemf(1, "/%s.predialog", cmd[udpc_config.modeIdx]);
  udpc_makeDevices(); /* Call this again, just in case the predialog script
		       * created any new devices */

  while(1) {
    int c = getopt( argc, argv, "tS:");
    if (c == EOF) break;
    switch (c) {
    case 't':
      udpc_testMode = 1;
      break;
    case 'S':
      close(2);
      open(optarg, O_WRONLY|O_CREAT|O_APPEND, 0644);
      break;
    case '?':
#ifndef NO_BB
      bb_show_usage();
#endif
      break;
    }
  }

  /* Allow rebooting via ctrl-alt-del */
  set_parm("/proc/sys/kernel/ctrl-alt-del", 1);

  /* Set the memory buffers to a value large enough for udpcast */
  set_parm("/proc/sys/net/core/wmem_max", 1048576);
  set_parm("/proc/sys/net/core/wmem_default", 1048576);
  set_parm("/proc/sys/net/core/rmem_max", 1048576);
  set_parm("/proc/sys/net/core/rmem_default", 1048576);



  /* Switch off printing of kernel messages to console */
  klogctl(6,0,0);

  /* Zero out config */
  udpc_clearConfig(&udpc_config, desc);
  udpc_loadConfigEnv(&udpc_config, desc, 0);
  udpc_loadConfig(AT_FDCWD, "udpcfg.txt", &udpc_config, desc, 0);
  
  /* maybe we have a floppy? */
  for(i=0; i<sizeof(floppyModules)/sizeof(floppyModules[0]); i++)
    udpc_modprobe(floppyModules[i], "");
  udpc_haveFloppy=!udpc_mountFloppy(0, NULL);
  if(udpc_haveFloppy) {
    loadStuffFromRemovable("/floppy");
    udpc_umountFloppy();
  }

#ifdef CONFIG_UDPCDIALOG_USB
  /* ... or a USB stick */

  udpc_haveUsb = !udpc_mountUsbRo();
  if(udpc_haveUsb) {
    loadStuffFromRemovable("/removable");
    udpc_umountUsb();
  }
#endif

  init_dialog();
  mainDialog();
  end_dialog();

  {
      int r;
      const char *base = cmd[udpc_config.modeIdx];
      close(2);
      r=dup(1);

      systemf(1, "/pre");
      systemf(1, "/%s.pre", base);
      r=systemf(0, "%s%s --file %s -P %d -i %s %s %s",
		base, compr[udpc_config.modeIdx * 3 + udpc_config.comprIdx],
		udpc_config.disk,
		udpc_config.port ? udpc_config.port : 9000,
		udpc_iface,
		udpc_config.udpcParam,
		udpc_config.modeIdx ? udpc_config.udpsParam : udpc_config.udprParam);
      systemf(1, "/%s.post %d",base, r);
      systemf(1, "/post %d", r);
  }
  return 0;
}
