#ifndef UDPCDIALOG_H
#define UDPCDIALOG_H

#ifndef USE_DESKTOP
#define USE_DESKTOP(x) /**/
#endif

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>


#ifdef NO_BB

#define CONFIG_UDPCDIALOG_USB 1
#define CONFIG_UDPCDIALOG_PXESERVER 1

#define ENABLE_FEATURE_SYSLOG 0
#define ENABLE_FEATURE_CROND_D 0
#define ENABLE_FEATURE_PS_ADDITIONAL_COLUMNS 0
#define ENABLE_FEATURE_SHOW_THREADS 0
#define ENABLE_FEATURE_TOPMEM 0
#define ENABLE_FEATURE_TOP_SMP_PROCESS 0
#define ENABLE_KILLALL 0
#define ENABLE_PGREP 0
#define ENABLE_PKILL 0
#define ENABLE_PIDOF 0
#define ENABLE_SELINUX 0
#define ENABLE_SESTATUS 0
#define IF_ECHO(x) /**/
#define IF_TEST(x) /**/
#define IF_KILL(x) /**/
#define IF_LS(x) /**/
#define IF_CHOWN(x) /**/
#define IF_PRINTF(x) /**/
#define IF_FEATURE_SHOW_THREADS(x) /**/
#define IF_SELINUX(x) /**/

#  define PMSG(...) fprintf(stderr, __VA_ARGS__)

#else
# include <autoconf.h>

# ifdef CONFIG_UDPCDIALOG_VERBOSE
#  define PMSG(...) fprintf(stderr, __VA_ARGS__)
# else
#  define PMSG(...) /**/
# endif

#endif

/* const-correct version of execv
 * Workaround against bug in SUSv3 specification:
 *   http://freebsd.monkey.org/freebsd-hackers/200307/msg00377.html
 */
static inline int cc_execvp(const char *cmd, const char * const * argv) {
        return execvp(cmd, (char * const *) argv);
}

/* Allow to pass char * const * as char const * const * 
 * See http://gcc.gnu.org/bugzilla/show_bug.cgi?id=49748
*/
static inline char const * const * ccast(char * const *p) {
	return (char const * const *) p;	
}

int udpc_spawn(int in, int out, const char **str);
int udpc_shell_spawn(int in, int out, const char *fmt, ...);
int udpc_display_error(void);
int udpc_display_output(void);

char *udpc_centerString(char *buffer, const char *value);
int udpc_findEntry(int nItems, const char * const *items, char *search);


int udpc_choseFromList(int needInput,
		       int title, const char *msg, 
		       int height, int width, int menu_height, 
		       unsigned int item_no, const char * const * items,
		       int keyCol, char *value, int maxLength,
		       int (*freeForm)(int));

void udpc_alert(const char *fmt, ...);
int udpc_info(const char *fmt, ...);
int udpc_msgbox(const char *title, const char *fmt, ...);
void udpcd_fatal(const char *fmt, ...);
char *udpc_format(int *size, const char *fmt, ...);
void udpc_fileNotFound(const char *filename);


void udpc_clearKlog(void);
int udpc_displayKlog(int isRed);

/* User interface configuration */
extern unsigned int udpc_testMode;
int udpc_uiConfig(int forceManual, int dir, void *data);

/* Net driver configuration */
int udpc_configureNetDriver(int forceManual, int dir, void *data);
int udpc_loadModulesFromFloppy(const char *root);

long long udpc_getMac(const char *ifname, char *ret);

/* IP Address configuration */
int udpc_configureNetSettings(int forceManual, int dir, void *data);

/* Disk drivers (SCSI/SATA), if needed */
int udpc_configureDiskDriver(int forceManual, int dir, void *data);

/* Udpcast configuration */
int udpc_configureUdpcast(int forceManual, int dir, void *data);

/* Start PXE server */
int udpc_configurePxeServer(int forceManual, int dir, void *data);



#define LANGSIZE 10
#define KMAPSIZE 10
#define MODSIZE 20
#define MODPARAMSIZE 60
#define IFNAMESIZE 12
#define DISKNAMESIZE 60
#define MODESIZE 4
#define COMPRSIZE 5
#define UDPCPARAMSIZE 1989

extern struct config {
  int automatic;	/* Automatic setup */

  char language[LANGSIZE];	/* Language of setup program */

  char keymap[KMAPSIZE];	/* Keyboard layout */

  int netAutoconfig;	/* Autoconfigure the network using PCI info */
  char netModule[MODSIZE];	/* Name of network module */
  char netParam[MODPARAMSIZE];	/* Parameter of network module */
  int netParamIsSet;	/* Has the network param been set? */
  char defNic[IFNAMESIZE]; /* Default interface name */

  int enableDiskmodule;		/* Should we enable a disk module? */
  char diskModule[MODSIZE];	/* Name of disk module */
  char diskParam[MODPARAMSIZE];	/* Parameter of disk module */
  int diskParamIsSet;	/* Has the disk param been set? */

  int doDhcp;		/* Configure IP Adress using DHCP */

  struct in_addr ip;	/* IP Address of this node */
  int ipIsSet;		
  struct in_addr netmask;	/* Netmask */
  int netmaskIsSet;	/* Is the netmask set? */


  int port;		/* network port to use */
  int portIsSet;	/* is the port set?*/

  char disk[DISKNAMESIZE];	/* Name of disk where to copy from/to */

  char mode[MODESIZE];	/* Mode: receiver/sender compressed/uncompressed */
  int modeIdx;

  char compr[COMPRSIZE];	/* Mode: receiver/sender compressed/uncompressed */
  int comprIdx;

  char udpcParam[UDPCPARAMSIZE];	/* Additional parameters for UDPCast */
  int udpcParamIsSet;

  char udprParam[UDPCPARAMSIZE];	/* Additional parameters for
					   udp-receiver */
  int udprParamIsSet;

  char udpsParam[UDPCPARAMSIZE];	/* Additional parameters for
					   udp-sender */
  int udpsParamIsSet;

  int doPxe;				/* Start a PXE server */

} udpc_config;

#define MENUSIZE(x) sizeof(x)/sizeof(char*)/2
#define MENU(x) MENUSIZE(x),x

extern char yesBuffer[];
extern char noBuffer[];
extern char okBuffer[];
extern char cancelBuffer[];

int udpc_mountFloppy(int alwaysAsk, const char *askMsg);
int udpc_umountFloppy(void);
extern int udpc_haveFloppy;

int udpc_modprobe(const char *module, const char *params);

int udpc_makeDevices(void);
int udpc_fileExists(const char *path);

char *nt_strncpy(char *dest, const char *src, size_t n);

int udpc_countLines(const char *path, const char *prefix, int prefixLen);

/* const char* getInterface(void);*/
int forAllNetInterfaces(int(*fn)(const char*), int remember);

/* Boolean dialog */
int udpc_askBoolean(int title, const char *prompt,
		    int height, int width,
		    int *value, int forceManual);

/* Return values for dialog series: */
/* Move forward in parent dialog series */
#define DIALOG_FORWARD 0

/* Move back in parent dialog series */
#define DIALOG_BACK -1

/* Skip this entry (i.e. keep moving in the direction we were moving
   in the parent dialog series */
#define DIALOG_SKIP -2

/* Also exit the parent dialog series, and move forward within grand-parent */
#define DIALOG_EXIT -3

/* Repeat this dialog element again */
#define DIALOG_REPEAT -4

/* Start over from beginning of this dialog series */
#define DIALOG_STARTOVER -5

/**
 * Configuration method.
 * dir = 1 for going forwards, and 0xff for going backwards
 * forceManual if manual mode should be forced (without this, entries
 * for which we already have an answer are skipped)
 * return value is described as above
 */
typedef int(*configFn_t)(int forceManual, int dir, void *data);

/**
 * Ask a series of questions, each implemented by a configFn function
 * fn is a table of question functions to be invoked
 * dir is 1 for going forwards, and 0xff for going backwards
 * forceManual if manual mode should be forced
 */
int dialogSeries(configFn_t *fn, int forceManual, int dir, void *data);

int udpc_mountUsbRo(void);
int udpc_umountUsb(void);

extern char *udpc_iface;
extern int udpc_haveUsb;

#define UNUSED __attribute__((unused))
#include "libbb.h"

#ifdef NO_BB
#undef errno
#define errno 0

#endif
#endif
