/*
 *  udpc_diskdriver.c -- disk driver load dialog for udpcast boot system
 *
 *  AUTHOR: Alain Knaff (alain@knaff.lu)
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif
#include "udpc_lang.h"
#include "dialogExt.h"
#include "udpc_dialog.h"
#include "udpc_dev.h"
#include "libbb.h"

/* ===============================================================
 * Setting of disk module configuration
 * =============================================================== */
static unsigned int diskPciClasses[] = { 0x100, 0x101, 0x104, 0x180 };
static unsigned int diskUsbClasses[] = { 0x08 };

static const char *diskPrefixes[] = { "drivers/scsi/", "drivers/ata/", "drivers/usb/storage/" };

static modules_t diskModules = {
  .pciClasses = diskPciClasses,
  .nPciClasses = sizeof(diskPciClasses) / sizeof(diskPciClasses[0]),
  .usbClasses = diskUsbClasses,
  .nUsbClasses = sizeof(diskUsbClasses) / sizeof(diskUsbClasses[0]),
  .prefixes = diskPrefixes,
  .nPrefixes = sizeof(diskPrefixes) / sizeof(diskPrefixes[0]),
  .title = TXT_TITLE_DISKDRV,
  .initFn = NULL,
  .isReadyFn = NULL,
  .choiceListPtr = NULL,
  .paramPrompt = TXT_ENTER_PARAMS,
  .prompt = TXT_LOAD_DISK_MODULE,
};

static int setPromptDiskmodule(int forceManual)
{
  return udpc_askBoolean(diskModules.title, MSG(TXT_LOAD_DISK_YESNO), 6, 50, 
			 &udpc_config.enableDiskmodule, forceManual);
}

int udpc_configureDiskDriver(int forceManual, int dir, void *data UNUSED)
{
  int r;

  while(1) {
    if(setPromptDiskmodule(forceManual) < 0)
      return DIALOG_BACK;
    if(udpc_config.enableDiskmodule==0)
      return DIALOG_FORWARD;

    diskModules.modName       = udpc_config.diskModule;
    diskModules.modParams     = udpc_config.diskParam;
    diskModules.modParamIsSet = &udpc_config.diskParamIsSet; 
    r = udpc_configureDriver(&diskModules, forceManual, dir);
    if(r == DIALOG_BACK)
      continue; /* Loop back to the YES/NO dialog */

    if(r == 0) {
      int k=0;
      udpc_clearKlog();
      if(udpc_modprobe("sd_mod", "")) {
	if(!udpc_displayKlog(1))
	  /* If no kernel error, display applicative error */
	  udpc_display_error();
      }
      /* Call this again to see any new devices */
      while(k < 100 && !udpc_makeDevices()) {
	usleep(30000);
	k++;
      }
    }
    return r;
  }
}
