/*
 *  udpc_conffile.c -- network driver load dialog for udpcast boot system
 *
 *  AUTHOR: Alain Knaff (alain@knaff.lu)
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include "udpc_lang.h"
#include "udpc_dialog.h"
#include "udpc_dev.h"
#include "libbb.h"

/* ===============================================================
 * Setting of net module configuration
 * =============================================================== */
static unsigned int netPciClasses[] = { 0x200, 0x680 };
static unsigned int netUsbClasses[] = { 0x02, 0x0A, 0xE0, 0xEF };

static const char *netPrefixes[] = { "drivers/net/" };

/* Probe for PCMCIA cards */
static int udpc_probecards(void)
{
  return system("/bin/probecards");
}

static int none(const char *ifc) {
  return udpc_getMac(ifc, NULL) == -1;
}

static int net_isReady(void) {
  /* we have a mac address ==> ok */
  return !forAllNetInterfaces(none, 0);
}

static modules_t netModules = {
  .pciClasses = netPciClasses,
  .nPciClasses = sizeof(netPciClasses) / sizeof(netPciClasses[0]),
  .usbClasses = netUsbClasses,
  .nUsbClasses = sizeof(netUsbClasses) / sizeof(netUsbClasses[0]),
  .prefixes = netPrefixes,
  .nPrefixes = sizeof(netPrefixes) / sizeof(netPrefixes[0]),
  .title = TXT_TITLE_NETDRV,
  .initFn = udpc_probecards,
  .isReadyFn = net_isReady,
  .choiceListPtr = NULL,
  .paramPrompt = TXT_ENTER_PARAMS,
  .prompt = TXT_LOAD_NET,
};


int udpc_configureNetDriver(int forceManual, int dir, void *data UNUSED)
{
  netModules.modName       = udpc_config.netModule;
  netModules.modParams     = udpc_config.netParam;
  netModules.modParamIsSet = &udpc_config.netParamIsSet;
  return udpc_configureDriver(&netModules, forceManual, dir);
}
