/*
 *  udpc_conffile.c -- PXE server dialog for udpcast boot system
 *
 *  AUTHOR: Alain Knaff (alain@knaff.lu)
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include "udpc_dialog.h"
#include "libbb.h"
#include "dialog.h"
#include "udpc_lang.h"
#include "udpc_conffile.h"

/* ===============================================================
 * Start PXE Server?
 * =============================================================== */

static int pxe_running = 0;

static int configurePxeYesNo(int forceManual)
{
  if(udpc_askBoolean(TXT_TITLE_PXESERVER, MSG(TXT_START_PXE), 5, 50,
		     &udpc_config.doPxe, forceManual) < 0)
    return -1;
  if(udpc_config.doPxe) {
    unsigned int ip = ntohl(udpc_config.ip.s_addr);
    unsigned int mask = ntohl(udpc_config.netmask.s_addr);
    unsigned int begin;
    struct in_addr beginAddr, endAddr;
    unsigned int end;
    extern struct configDesc desc[];

    udpc_saveConfig("/tmp/udpcfg.txt", &udpc_config, desc);

    begin = ip & mask;
    end = begin | ~mask;

    if(ip - begin < end - ip) {
      begin = ip;
    } else {
      end = ip;
    }
    endAddr.s_addr = htonl(end - 1);
    beginAddr.s_addr = htonl(begin + 1);

    setenv("IFACE", udpc_iface, 1);
    setenv("IP", inet_ntoa(udpc_config.ip), 1);
    setenv("NETMASK", inet_ntoa(udpc_config.netmask), 1);
    setenv("BEGIN", inet_ntoa(beginAddr), 1);
    setenv("END", inet_ntoa(endAddr), 1);

    if(system("/bin/startPxeServer"))
      return -1;
    else
      pxe_running = 1;
  }
  return 0;
}

int udpc_configurePxeServer(int forceManual, int dir UNUSED, void *data UNUSED)
{
  struct stat statbuf;
  int ret;

  if(pxe_running ||
     udpc_config.modeIdx == 0 ||
     udpc_config.doDhcp) {
    /*    PXE is already running => as we can't stop it, skip this module */

    /* or we are receiver => it makes more sense to make sender the
          DHCP server as well, so skip*/

    /* or, we got our own address through DHCP */

    return DIALOG_SKIP;
  }

  /* If we boot from CD, we need to try mounting again, as the SCSI
   * host driver for the CD filesystem might not yet have been available
   * at udpcdialog start (... if relevant driver was only loaded later during
   * disk driver dialog...)
   */
  if(!udpc_haveUsb && lstat("/cd", &statbuf) == 0) {
    udpc_haveUsb = !udpc_mountUsbRo();
  }

  if(!udpc_haveUsb || lstat("/pxe", &statbuf) < 0)
    /* Error: No PXE directory */
    return DIALOG_SKIP;

  if(udpc_config.automatic != 1)
    forceManual=1;

  ret = configurePxeYesNo(forceManual);
  udpc_umountUsb();
  return ret;
}
