/*
 *  udpc_conffile.c -- UI config tool for udpcast boot system
 *
 *  AUTHOR: Alain Knaff (alain@knaff.lu)
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif
#include <errno.h>
#include "udpc_dialog.h"
#include "libbb.h"
#include "dialog.h"
#include "udpc_lang.h"
#include "udpc_conffile.h"


unsigned int udpc_language=0;
unsigned int udpc_testMode=0;

static const char *languages[] = {
  "US", "English",
  "DA", "Dansk",
  "DE", "Deutsch",
  "FR", "Fran\207ais",
  "LU", "L\211tzebuerg\211sch",
};

static const char *keymaps[] = {
  "US",	"United States",
  "SG", "Swiss German",
  "BE", "Belgian",
  "FR", "French",
  "DE", "German"
};

char yesBuffer[]    ="     ";
char noBuffer[]     ="     ";
char okBuffer[]     ="      ";
char cancelBuffer[] ="      ";

/* Make sure some frequent language-specific texts are appropriately centered
   needs to be called after each language change */
static void udpc_centerTextsForLanguage(void)
{
  udpc_centerString(yesBuffer,MSG(TXT_YES));
  udpc_centerString(noBuffer,MSG(TXT_NO));

  udpc_centerString(okBuffer,MSG(TXT_OK));
  udpc_centerString(cancelBuffer,MSG(TXT_CANCEL));
}


static int setLanguage(int forceManual, int dir UNUSED, void *data UNUSED)
{
  int r;
  if(udpc_config.automatic != 1)
    forceManual=1;
  r=udpc_findEntry(MENU(languages), udpc_config.language);
  if(r==-1) {
    forceManual=1;
    udpc_language=0;
  } else {
    udpc_language=r;
  }
  udpc_centerTextsForLanguage();

  r=udpc_choseFromList(forceManual,
		       TXT_TITLE_UI, MSG(TXT_CHOOSE_LANGUAGE),
		       20, 35, 10, MENU(languages),
		       0, 
		       udpc_config.language, sizeof(udpc_config.language),
		       NULL);
  if(r < 0)
    return DIALOG_BACK;
  udpc_language=r;
  udpc_centerTextsForLanguage();
  return DIALOG_FORWARD;
}

/* ======================================================
 * Setting of the keymap
 * ====================================================== */
static const char *loadkmap_cmd[] = { "loadkmap", NULL };

static int setKeymap(int needInput, int dir UNUSED, void *data UNUSED)
{
  while(1) {
    int r=udpc_choseFromList(needInput,
			     TXT_TITLE_UI, MSG(TXT_MENU_KEYMAP),
			     20, 30, 10, MENU(keymaps), 0, 
			     udpc_config.keymap, sizeof(udpc_config.keymap),
			     NULL);
    if(r < 0)
      return DIALOG_BACK;
    if(!udpc_testMode) {
      char filename[] = "/keymaps/XX.bkm";
      int f;
      filename[9] = udpc_config.keymap[0];
      filename[10] = udpc_config.keymap[1];
      f = open(filename, O_RDONLY);
      if(f < 0) {
	udpc_fileNotFound(filename);
      } else {
	if(!udpc_testMode && udpc_spawn(f, -1, loadkmap_cmd)) {
	  close(f);
	  udpc_display_error();
	} else {
	  close(f);
	  return DIALOG_FORWARD;
	}
      }
    }
    needInput=1;
  }
}

static configFn_t menu[] = {
  setLanguage,
  setKeymap,
  NULL
};


int udpc_uiConfig(int forceManual, int dir, void *data UNUSED)
{
  udpc_centerTextsForLanguage();
  return dialogSeries(menu, forceManual, dir, NULL);
}
