#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/wait.h>

#include "libbb.h"

#define UNUSED __attribute__((unused))

char **environ;

//config:config USBLOADER
//config:        bool "USBLOADER"
//config:        default y
//config:        help
//config:          USB Loader

//kbuild:lib-$(CONFIG_USBLOADER) += usbloader.o

//applet:IF_USBLOADER(APPLET_ODDNAME(init, usbloader, BB_DIR_USR_BIN, BB_SUID_DROP, init))

//usage:#define usbloader_trivial_usage
//usage:      ""
//usage:#define usbloader_full_usage
//usage:      "Small program to chainload main initrd from USB\n\n"

/* const-correct version of execv
 * Workaround against bug in SUSv3 specification:
 *   http://freebsd.monkey.org/freebsd-hackers/200307/msg00377.html
 */
static int cc_execv(const char *cmd, const char * const * argv) {
	return execv(cmd, (char * const *) argv);
}

static int my_spawn(const char *cmd, const char *const argv[]) {
	int status;
	switch(fork()) {
		case 0: /* child */
			cc_execv(cmd, argv);
			fprintf(stderr, "Could not exec %s\n", cmd);
			exit(1);
		case -1:
			fprintf(stderr, "Could not fork\n");
			return -1;
		default:
			wait(&status);
			return status;
	}
	
}

static int my_popen(const char *cmd, const char *const argv[],
		    int in, int out) {
	int pid;

	switch((pid=fork())) {
		case 0: /* child */
			if(in != 0) {
				close(0);
				dup(in);
				close(in);
			}

			if(out != 1) {
				close(1);
				dup(out);
				close(out);
			}

			cc_execv(cmd, argv);
			fprintf(stderr, "Could not exec %s\n", cmd);
			exit(1);
		case -1:
			fprintf(stderr, "Could not fork\n");
			return -1;
		default:
			return pid;
	}
}

static const char * const modules[] = {
	"mod/usbcore.ko",

	"mod/ehci-hcd.ko",

	"mod/ohci-hcd.ko",
	"mod/uhci-hcd.ko",

	"mod/scsi_mod.ko",
	"mod/sd_mod.ko",

	"mod/usb-libusual.ko",
	"mod/usb-storage.ko",

	"mod/fat.ko",
	"mod/vfat.ko",

	"mod/nls_cp850.ko",

	NULL
};

int usbloader_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
int usbloader_main(int argc UNUSED, char **argv UNUSED) {
	const char *insmod_cmd[] = { "insmod", NULL, NULL, NULL };
	const char *const mount_cmd[] = { "mount", "-r", "-t", "vfat", "-o",
				    "codepage=850,iocharset=cp850", "/sda1",
				    "/usb", NULL };
	const char *const decompress_cmd[] = { "zcat",
					       "/usb/boot/udpcast/initrd",
					       NULL };

	const char *const cpio_cmd[] = { "cpio", "-i", NULL };

	const char *const umount_cmd[] = { "umount", "/usb", NULL };

	const char *const init_cmd[] = { "init", NULL };
					 
	pid_t cpio_pid;
	const char * const * ptr;
	int pipefd[2];
	int status;

	if(rename("/init", "/busybox") < 0) {
		perror("rename");
	}

	printf("USB loader\n");
	for(ptr = modules; *ptr; ptr++) {
		insmod_cmd[1] = *ptr;
		my_spawn("busybox", insmod_cmd);
	}

	sleep(2);

	if(mknod("/sda1", S_IFBLK, makedev(8, 1)) < 0) {
		perror("mknod");
	}

	my_spawn("busybox", mount_cmd);

	unlink("/sda1");

	printf("Unpacking initrd\n");

	if(pipe(pipefd) < 0) {
		perror("pipefd");
	}
	my_popen("busybox", decompress_cmd, 0, pipefd[1]);
	cpio_pid = my_popen("busybox", cpio_cmd, pipefd[0], 1);
	close(pipefd[0]);
	close(pipefd[1]);

	waitpid(cpio_pid, &status, 0);

	printf("Done unpacking initrd\n");
	unlink("/busybox");
	my_spawn("/bin/busybox", umount_cmd);
	
	cc_execv("/init", init_cmd);
	perror("execve");
	return -1;
}
